#!/bin/sh
#**************************************************************** {COPYRIGHT-TOP-RM} ***
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#**************************************************************** {COPYRIGHT-END-RM} ***
#***********************************************************************
# Generic syntax help.
#
TGM_ROOT="/TDS/EventService"
PATH=$PATH:/usr/ucb

syntax()
 {
   echo
   echo "unix.setup.sh"
   echo
   echo "  syntax:   unix.setup.sh [ -? ]"
   echo
   echo "  usage:    you must be root to run this script"
   echo
   echo
 }


for i in $@
do
  if [ "$i" = "-?" ]
   then
     syntax
     exit 0
   else
     echo
     echo .... ERROR - invalid flag \"$i\" entered.
     syntax
     exit 1
  fi
done

echo "Tivoli Business Systems Manager server setup starting"

#
# UNIX unique processing
#
if [ "`uname`" = "AIX" ] || [ "`uname`" = "SunOS" ] ; then
  if [ "`whoami`" != "root" ] ; then
   echo
   echo ".... You must be the super-user (root) to run this script"
   syntax
   exit 1
  fi

  echo ".... Setting Tivoli environment variables."

  if [ -n "$TSERVER_ETC" ] ; then
    if [ -f $TSERVER_ETC/setup_env.sh ] ; then
      . $TSERVER_ETC/setup_env.sh
    else
      echo ".... Unable to find file $TSERVER_ETC/setup_env.sh"
      echo ".... Tivoli enviroment variables can not be set."
      exit 1
    fi
  elif [ -f /etc/Tivoli/setup_env.sh ] ; then
   . /etc/Tivoli/setup_env.sh
  else
    echo ".... Unable to find file /etc/Tivoli/setup_env.sh"
    echo ".... Tivoli enviroment variables can not be set."
    exit 1
  fi

  #
  # Detemine the OS and set OS-specific paths, falling back to defaults
  # for AIX.
  #

  arch=`uname -s 2> /dev/null`
  if [ "$arch" = "" ] ; then
   echo ".... Unable to determine your OS."
   exit 1
  fi
  archrev=`uname -r | cut -c1`
  if [ "$arch" = "AIX" ] ; then
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1=/usr/lib/X11/app-defaults
   RESBASE2=/usr/lib/X11/
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/bin/mkgroup
  elif [ "$arch" = "SunOS" -a "$archrev" = "5" ]; then
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1="0"
   RESBASE2="0"
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/groupadd
  else
   NLSBASE1=/usr/lib/nls/msg
   NLSBASE2=/usr/dt/lib/nls/msg
   RESBASE1="0"
   RESBASE2="0"
   RESBASE3=/usr/dt/app-defaults/
   MK_GROUP=/usr/sbin/mkgroup
  fi
  export CATFILE NLSBASE1 NLSBASE2 RESBASE1 RESBASE2 RESBASE3 MK_GROUP

  echo .... Creating $arch message catalog links
  if [ "$arch" = "AIX" ] ; then

   for THELOOP in "4" "5" ; do
     if [ "$THELOOP" = "4" ] ; then
       THELANG=En_US
       NEWLANG=En_US.IBM-850
     elif [ "$THELOOP" = "5" ] ; then
       THELANG=en_US
       NEWLANG=en_US.ISO8859-1
     else
       THELANG=ja_JP
       NEWLANG=ja_JP.IBM-eucJP
     fi

     for LINKDIR in "$BINDIR$TGM_ROOT/msg_cat" ; do
       if [ -d "$LINKDIR/$THELANG" ] ; then
         if [ ! -d "$LINKDIR/$NEWLANG" ] ; then
           mkdir "$LINKDIR/$NEWLANG"
         fi

         cd "$LINKDIR/$THELANG"

         for CATFILE in `ls` ; do
           if [ -f "$LINKDIR/$NEWLANG/$CATFILE" ] || [ -h "$LINKDIR/$NEWLANG/$CATFILE" ] ; then
             rm "$LINKDIR/$NEWLANG/$CATFILE"
           fi

           echo .... Creating symbolic links for message catalogs $LINKDIR/$NEWLANG/$CATFILE
           ln -s "$LINKDIR/$THELANG/$CATFILE" "$LINKDIR/$NEWLANG/$CATFILE"
         done
       fi
     done
   done

  else
    echo "WARNING:  Architecture $arch is not supported.  Message catalogs will not be linked"
  fi

  #
  # Symlink all of the .cat files to the specified locales.
  #
  echo ".... Creating system message catalog links."

  cd "$BINDIR$TGM_ROOT/msg_cat"

  for TLANG in `ls` ; do
    for NLSBASE in $NLSBASE1 $NLSBASE2 ; do
      if [ -d "$NLSBASE" ] ; then
        if [ -d "$NLSBASE/$TLANG" ] ; then
          cd "$BINDIR$TGM_ROOT/msg_cat/$TLANG"
          for CATFILE in `ls` ; do
            if [ -f "$NLSBASE/$TLANG/$CATFILE" ] || [ -h "$NLSBASE/$TLANG/$CATFILE" ] ; then
              rm "$NLSBASE/$TLANG/$CATFILE"
            fi

            echo .... Creating symbolic links for message catalogs $NLSBASE/$TLANG/$CATFILE
            ln -s "$BINDIR$TGM_ROOT/msg_cat/$TLANG/$CATFILE" "$NLSBASE/$TLANG/$CATFILE"
          done
        else
          echo .... The system $NLSBASE/$TLANG message catalogs are not installed on this system.
          echo .... Message catalog links for $TLANG will not be set.
          echo .... If in the future the system $TLANG catalogs are installed, then
          echo .... unix.setup.sh should be re-run to setup this link.
        fi
      fi
    done
  done

  #
  # Make symlinks from Tivoli msg_cat to TGM's messages.
  #
  echo ".... Link message catalogs from Tivoli msg_cat directory."

  cd "$BINDIR"
  cd ../..

  if [ -d "msg_cat" ] ; then
    cd "msg_cat"

    TIV_MSGCAT=`pwd`

    for TLANG in `ls` ; do
      if [ -d "$BINDIR$TGM_ROOT/msg_cat/$TLANG" ] ; then

        cd "$BINDIR$TGM_ROOT/msg_cat/$TLANG"

        for CATFILE in `ls` ; do
          #
          # Remove the symlink if it exists.
          #
          if [ -h "$TIV_MSGCAT/$TLANG/$CATFILE" ] ; then
            echo ".... Removing $TIV_MSGCAT/$TLANG/$CATFILE from Tivoli message catalog."
            rm "$TIV_MSGCAT/$TLANG/$CATFILE"
          fi

          echo .... Creating symbolic links for message catalogs $TIV_MSGCAT/$TLANG/$CATFILE
          ln -f -s "$BINDIR$TGM_ROOT/msg_cat/$TLANG/$CATFILE" "$TIV_MSGCAT/$TLANG/$CATFILE"
        done
      else
        echo ".... Links from Tivoli $TIV_MSGCAT/$TLANG to msg_cat were not built."
        echo ".... The $BINDIR$TGM_ROOT/msg_cat/$TLANG messages are not installed."
        echo ".... This is not an error."
      fi
    done
  fi

  #
  # Check for and symbolically link libraries.
  #
  echo ".... Checking for Open Class and Exit Libraries."

  #
  # Adjusting ipfx ipfx2 libraries
  #
  if [ -f "$BINDIR$TGM_ROOT/lib/libipfx2.a" ] ; then
    if [ ! -f "$BINDIR$TGM_ROOT/lib/libipfx.a" ] ; then
      cp "$BINDIR$TGM_ROOT/lib/libipfx2.a" "$BINDIR$TGM_ROOT/lib/libipfx.a"
    fi
  fi

  for LIBFILE in `ls "$BINDIR$TGM_ROOT/lib"` ; do
    ALIBFILE="$BINDIR$TGM_ROOT/lib/$LIBFILE"

    if [ -f "$ALIBFILE" ] && [ "$LIBFILE" != "librasgui.so.1" ] && [ "$LIBFILE" != "librasgui.so" ] && [ "$LIBFILE" != "libC.a" ] ; then
      if [ -h "/usr/lib/$LIBFILE" ] ; then
        echo ".... Checking link for $LIBFILE"
        echo `ls -al "/usr/lib/$LIBFILE"` | grep GEM > /dev/null
        if [ $? = 0 ] ; then
          echo ".... Removing link for /usr/lib/$LIBFILE"
          rm "/usr/lib/$LIBFILE"
        else
          echo `ls -al "/usr/lib/$LIBFILE"` | grep TDS > /dev/null
          if [ $? = 0 ] ; then
            echo ".... Removing link for /usr/lib/$LIBFILE"
            rm "/usr/lib/$LIBFILE"
          fi
        fi
      fi

      if [ ! -f "/usr/lib/$LIBFILE" ] ; then
        if [ ! -h "/usr/lib/$LIBFILE" ] ; then
          echo ".... Creating link for $LIBFILE"
          ln -s "$BINDIR$TGM_ROOT/lib/$LIBFILE" "/usr/lib/$LIBFILE"
        fi
      fi
    fi

  done

  #
  # Generate symbolic links to versioned shared libraries for Solaris and
  # other ELF systems.
  #
  for LIBFILE in "$BINDIR$TGM_ROOT/lib/lib*.so.1"
  do
    LINK=`echo "$LIBFILE" | sed 's/\.so\.1$/.so/'`
    BLINK=`basename "$LINK"`
    echo ".... Checking link for `basename $LIBFILE`"
    if [ ! -z "$LINK" -a ! -h "$LINK" ] ; then
      echo ".... Setting link for `basename $LIBFILE`"
      ln -s -f `basename "$LIBFILE"` "$LINK"
    fi

    echo ".... Checking link for /usr/lib/$BLINK"
    if [ ! -z "/usr/lib/$BLINK" -a ! -h "/usr/lib/$BLINK" -a "$BLINK" != "librasgui.so" ] ; then
      echo ".... Setting link for /usr/lib/$BLINK"
      ln -s -f "$LIBFILE" "/usr/lib/$BLINK"
    fi
  done

  #
  # Handle the special case of libipfx2 and libipfx
  #
  if [ -f "/usr/lib/libipfx2.a" ] ; then
    if [ ! -f "/usr/lib/libipfx.a" ] ; then
      ln -s "/usr/lib/libipfx2.a" "/usr/lib/libipfx.a"
    fi
  fi

  #
  # Copy tbsm_startd.sh
  #
  #date
  #echo ".... Checking for the startup script."

  #if [ -d "/etc/Tivoli" ] ; then
  #  if [ -f "$BINDIR$TGM_ROOT/bin/startd.sh" ] ; then
  #    if [ -f "/etc/Tivoli/tbsm_startd.sh" ] || [ -h "/etc/Tivoli/tbsm_startd.sh" ] ; then
  #      echo ".... Removing existing copy of /etc/Tivoli/tbsm_startd.sh"
  #      rm "/etc/Tivoli/tbsm_startd.sh"
  #    fi
  #
  #    echo ".... Copy tbsm_startd.sh to /etc/Tivoli/tbsm_startd.sh"
  #    cp "$BINDIR$TGM_ROOT/bin/startd.sh" "/etc/Tivoli/tbsm_startd.sh"
  #    chmod 755 "/etc/Tivoli/tbsm_startd.sh"
  #  else
  #    if [ -f "/etc/Tivoli/tbsm_startd.sh" ] ; then
  #      echo ".... Unable to find $BINDIR$TGM_ROOT/bin/tbsm_startd.sh."
  #      echo ".... /etc/Tivoli/tbsm_startd.sh will not be updated."
  #      echo ".... This is not an error."
  #    else
  #      echo ".... Unable to find $BINDIR$TGM_ROOT/bin/tbsm_startd.sh."
  #      echo ".... Unable to find /etc/Tivoli/tbsm_startd.sh."
  #      echo ".... This will prevent the daemon from being started"
  #      echo ".... by init at startup."
  #    fi
  #  fi
  #else
  #  echo ".... Unable to copy tbsm_startd.sh to /etc/Tivoli to tbsm_startd.sh."
  #  echo ".... This will prevent the daemon from being started"
  #  echo ".... by init at startup."
  #fi

  #
  # Copy stop to /etc/Tivoli/tbsm_stop.sh
  #

  #date
  #echo ".... Checking the link for the tbsm_stop script."

  #if [ -d "/etc/Tivoli" ] ; then
  #  if [ -f "/etc/Tivoli/tbsm_stop.sh" ] || [ -h "/etc/Tivoli/tbsm_stop.sh" ] ; then
  #    echo ".... Removing existing copy of /etc/Tivoli/tbsm_stop.sh"
  #    rm "/etc/Tivoli/tbsm_stop.sh"
  #  fi
  #
  #  echo ".... Copying stop to /etc/Tivoli/tbsm_stop.sh"
  #  ln -fs "$BINDIR$TGM_ROOT/bin/stop" "/etc/Tivoli/tbsm_stop.sh"
  #else
  #  echo ".... Unable to copy tbsm_stop to /etc/Tivoli."
  #  echo ".... This will prevent the daemon from being started"
  #  echo ".... by init at startup."
  #fi

  #
  # On AIX, update /etc/inittab to start the daemon
  #
  #if [ "$INTERP" = "aix4-r1" ] ; then
  # if [ -f "/etc/rc.tbsm" ] || [ -h "/etc/rc.tbsm" ] ; then
  #  echo "The daemon is already enabled to start.  No action required."
  # else
  #  if [ -f "/etc/Tivoli/tbsm_startd.sh" ] || [ -h "/etc/Tivoli/tbsm_startd.sh" ] ; then
  #    ln -fs "/etc/Tivoli/tbsm_startd.sh" "/etc/rc.tbsm"
  #    if   [ -f "/usr/sbin/mkitab" ] ; then
  #      /usr/sbin/mkitab "rctbsm:2:once:/etc/rc.tbsm >/dev/console 2>&1 # Tivoli BSM Task Server"
  #    elif [ -f "/etc/mkitab" ] ; then
  #      /etc/mkitab "rctbsm:2:once:/etc/rc.tbsm >/dev/console 2>&1 # Tivoli BSM Task Server"
  #    else
  #      mkitab "rctbsm:2:once:/etc/rc.tbsm >/dev/console 2>&1 # Tivoli BSM Task Server"
  #    fi
  #    echo "The daemon is enabled to start."
  #  fi
  # fi
  #fi

  #
  # On Solaris, add a link in /etc/rc3.d so that tbsm_startd.sh can be
  # started by init
  #
  #if [ "$INTERP" = "solaris2" ] ; then
  # if [ -f "/etc/Tivoli/tbsm_startd.sh" ] ; then
  #  if [ ! -h "/etc/rc3.d/S95tbsm" ] ; then
  #   ln -fs "/etc/Tivoli/tbsm_startd.sh" "/etc/rc3.d/S95tbsm"
  #   "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5942
  #  else
  #   "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5941
  #  fi
  # fi

  # if [ ! -h "/etc/rc0.d/K05tbsm" ] ; then
  #   ln -fs "/etc/Tivoli/tbsm_stop.sh" "/etc/rc0.d/K05tbsm"
  #   "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5943
  # else
  #   "$BINDIR$TGM_ROOT/bin/ihszmsgp" 2 5941
  # fi
  #fi

  #
  # Ensure mode for log and ipc directories
  #
  for DIR_NAME in ipc ipc/SEM ipc/QUEUES log ; do
    chmod 775 "$BINDIR$TGM_ROOT/$DIR_NAME"
  done

  #
  # Copy hostcmd to old server location - migration
  #
  if [ ! -d "$BINDIR/TDS/server" ] ; then
    mkdir "$BINDIR/TDS/server"
  fi

  if [ ! -d "$BINDIR/TDS/server/bin" ] ; then
    mkdir "$BINDIR/TDS/server/bin"
  fi

  if [ -f "$BINDIR/TDS/server/bin/hostcmd" ] ; then
    echo "$BINDIR/TDS/server/bin/hostcmd exists, this file will not be updated"
  else
    if [ ! -h "$BINDIR/TDS/server/bin/hostcmd" ] ; then
      ln -fs "$BINDIR$TGM_ROOT/bin/hostcmd" "$BINDIR/TDS/server/bin/hostcmd"
    fi
  fi

  #
  # end of UNIX unique processing
  #

else
  #
  # Intel unique processing
  #
  if [ -n "$TSERVER_ETC" ] ; then
    if [ -f "$TSERVER_ETC/setup_env.sh" ] ; then
      if [ -n "$windir" ] ; then
        . "$TSERVER_ETC/setup_env.sh"
      elif [ -n "$OS2_SHELL" ] ; then
        PATH="$BINDIR/bin:$BINDIR/tools:$PATH"
        export PATH
      fi
    fi
  else
    if [ -n "$windir" ] ; then
      WINOS=`echo "$windir" | awk /:/'{ print substr($1,4) }'`
      WINDRIVE=`echo "$windir" | awk /:/'{ print substr($1,1,2) }'`
      if [ -n "$WINOS" ] ; then
        if [ -f "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh" ] ; then
          . "$WINDRIVE/$WINOS/system32/drivers/etc/Tivoli/setup_env.sh"
        fi
      fi
    elif [ -n "$OS2_SHELL" ] ; then
      PATH="$BINDIR/bin:$BINDIR/tools:$PATH"
      export PATH
    fi
  fi

  if [ -n "$NLSPATH" ] ; then
    NLSPATH="$NLSPATH:$BINDIR$TGM_ROOT/msg_cat/%L/%N.cat"
  else
    NLSPATH="$BINDIR$TGM_ROOT/msg_cat/%L/%N.cat"
  fi
  export NLSPATH

  PATH="$PATH:$BINDIR$TGM_ROOT/bin:$BINDIR$TGM_ROOT/msg_cat/%L"
  export PATH

  #
  # Delete old dlls
  #
  for OLDFILE in cppwot3.dll ihsucrtl.dll ihsuob3i.dll ihsuor3u.dll ihsuou3i.dll ihsuov3.dll ; do
    if [ -f "$BINDIR/TDS/EventService/$OLDFILE" ] ; then
      echo "Removing: $BINDIR/TDS/EventService/$OLDFILE"
      rm "$BINDIR/TDS/EventService/$OLDFILE"
    fi
  done

  for OLDFILE in cppwot3.dll ihsucrtl.dll ihsuob3i.dll ihsuor3u.dll ihsuou3i.dll ihsuov3.dll ; do
    if [ -f "$BINDIR/TDS/EventService/bin/$OLDFILE" ] ; then
      echo "Removing: $BINDIR/TDS/EventService/bin/$OLDFILE"
      rm "$BINDIR/TDS/EventService/bin/$OLDFILE"
    fi
  done
     
  "$BINDIR$TGM_ROOT/bin/ihsxsrv" install AUTO "$BINDIR$TGM_ROOT/bin"

  #
  # Copy hostcmd to old server location - migration
  #
  if [ ! -d "$BINDIR/TDS/server" ] ; then
    mkdir "$BINDIR/TDS/server"
  fi

  if [ ! -d "$BINDIR/TDS/server/bin" ] ; then
    mkdir "$BINDIR/TDS/server/bin"
  fi

  if [ -f "$BINDIR/TDS/server/bin/hostcmd" ] ; then
    if [ -f "$BINDIR/TDS/server/bin/ihscacm.dll" ] ; then
      echo "$BINDIR/TDS/server/bin/hostcmd exists, this file will not be updated"
    else
      cp "$BINDIR$TGM_ROOT/bin/hostcmd" "$BINDIR/TDS/server/bin/hostcmd"
    fi
  else
    cp "$BINDIR$TGM_ROOT/bin/hostcmd" "$BINDIR/TDS/server/bin/hostcmd"
  fi
fi

echo "TGM setup complete."

