#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
# (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}

# File: archive/scripts/fo_failover.ksh
# SID: 1.27.6.3
# Date modified: 07/04/25 15:06:42
#
# Module Description:
#
# fo_failover.ksh - TBSM Failover command generation script
#
#   This script generate failover command in a text file.  
#
#   Note:  
#
#set -x


usage()
{
  if [ "$NLSMSG" == TRUE ]; then
    usage2
    return
  fi

  print "Usage: $0 [ -R | -F | -L | -Y | -T | -h ]"
  print "This script loads the Failover configuration, generates the failover command file,"
  print "and prepares TBSM servers for Failover execution."
  print ""
  print "  -R          Specifies reverse failover direction.  Default to forward"
  print "  -F          Forces failover direction - will not check failover direction"
  print "  -L          Launches the execution of the command file after it is generated"
  print "  -Y          Specifies NoSync option to leave out databases synchronization"
  print "  -T          Turns on Trace mode"
  print "  -h          Displays help info"
}

usage2()
{
  ASIShellMsg $FOMSG 802 $0
  ASIShellMsg $FOMSG 862
  ASIShellMsg $FOMSG 816
  ASIShellMsg $FOMSG 808
  ASIShellMsg $FOMSG 811
  ASIShellMsg $FOMSG 820
  ASIShellMsg $FOMSG 819
  ASIShellMsg $FOMSG 823
}

. load_fo_env

NoSync=FALSE
Launch=FALSE

while getopts "RTFLYh?" c; do
  case $c in

  R)  REVERSE=TRUE;;
  F)  ForceDirection=TRUE;;
  L)  Launch=TRUE;;
  Y)  NoSync=TRUE;;
  T)  Trace_flag=TRUE;;

  h) usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

#------------------------------------------------------------------------

#
#  usage: gen_startsvcs <site> either Target or Source
#
gen_startsvcs()
{
  setAllHost $1
  if [ $? != 0 ]; then
    return 1
  fi

  msgprint $INFO "Generating commands to start $1 services."
  msgprint2 223 $INFO $1

  write_cmd text "# Start $1 services"
  write_cmd dash

  # start WAS 
  #---------------------------------------
  for host in $(hostsin $HOST_WASServer)
  do
    write_cmd text "$host Opt startwas $PARM_WAS_WaitTime" 
  done

  # check SnaBase service on Event Handler Server 
  #------------------------------------------------
  for host in $(hostsin $HOST_EhServer) 
  do
    ##if $(print "$PARM_Communication_Type" | grep -iq -e "LU6"); then

    if [ "$HOST_SnaServer" != "" ]; then
      write_cmd text "$host Opt checksvc SnaBase" 
    fi
  done

  # start SQL Services
  #-------------------
  for host in $(hostsin $HOST_DatabaseServer $HOST_HistServer)
  do
    write_cmd text "$host Req svcstart MSSQLServer" 
    write_cmd text "$host Req svcstop SQLServerAgent" 
    write_cmd text "$host Req svcstart SQLServerAgent" 
  done

  # start ASIDBValidater
  #---------------------------------------
  for host in $(hostsin $HOST_DatabaseServer $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Req svcstart ASIDBValidater" 
  done

  # start ASIRemoteExecutionServer
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcstart ASIRemoteExecutionServer " 
  done

  # start ASIEnqueueProxyServer
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer $HOST_SsServer)
  do
    write_cmd text "$host Opt svcstart ASIEnqueueProxyServer " 
  done

  # start MVS UpLoadRule Server 
  #---------------------------------------
  write_cmd text ${HOST_UpLoadRuleServer:+$HOST_UpLoadRuleServer Opt svcstart ASIMVSUploadRuleSvc} 

  # start feed data processing
  #---------------------------------------
  write_cmd text "$HOST_DatabaseServer Req svcstart ASIStagedEventLoader " 

  # start AgentListener Server 
  #---------------------------------------
  write_cmd text ${HOST_AgentListener:+$HOST_AgentListener Opt svcstart ASIAgentListenerSvc} 
  write_cmd text ${HOST_AgentListener:+$HOST_DatabaseServer Opt svcstart ASIEventEnablement} 

  for host in $(hostsin $HOST_AgentListener $HOST_TGMTaskServer)
  do
    write_cmd text "$host Opt svcstart ASITaskServer " 
  done

  # start TGM Task Server 
  #---------------------------------------
  for host in $(hostsin $HOST_TGMTaskServer)
  do
    write_cmd text "$host Opt svcstart ASIEventEnablement " 
  done

  # start Common Listener 
  #---------------------------------------
  write_cmd text ${HOST_CommonListener:+$HOST_CommonListener Opt svcstart ASICommonlistener} 

  # start EIF Sender
  #---------------------------------------
  write_cmd text ${HOST_EIFSender:+$HOST_EIFSender Opt svcstart ASIEIFSender} 

  # start TEC Listener 
  #---------------------------------------
  write_cmd text ${HOST_TECListener:+$HOST_TECListener Opt svcstart ASITECListenerSvc} 

  # start SNA Server 
  #---------------------------------------
  write_cmd text ${HOST_SnaServer:+$HOST_SnaServer Req svcstart SnaServr} 

  # start MVS IPListener service
  #---------------------------------------
  write_cmd text "$HOST_DatabaseServer Opt svcstart ASIMVSIPListenerSvc " 

  # start MVS IP OS Listener service
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer)
  do
    write_cmd text "$host Opt svcstart ASIMVSIPOSListenerSvc " 
  done

  # start Sender Service Server 
  #---------------------------------------
  for host in $(hostsin $HOST_SsServer)
  do
    write_cmd text "$host Opt svc_Ss start $BATCHSVC_START_TIMER" 
  done

  # start Event Handler Server 
  #---------------------------------------
  for host in $(hostsin $HOST_EhServer) 
  do
    write_cmd text "$host Opt svc_Eh start $BATCHSVC_START_TIMER" 
  done

  # start IIS
  #---------------------------------------
  write_cmd text ${HOST_WebServer:+$HOST_WebServer Opt svcstart W3SVC} 

  # start XML Toolkit
  #---------------------------------------
  for host in $(hostsin $HOST_XMLToolkit) 
  do
    write_cmd text "$host Opt svcstart ASIXMLToolkitSvc" 
  done

  # start Health Monitor
  #---------------------------------------
  write_cmd text ${HOST_HmServer:+$HOST_HmServer Opt svcstart ASIHealthMonitor} 

  # update Health Monitor client
  #---------------------------------------
  for host in $(hostsin $HOST_HmClient) 
  do
  	write_cmd text ${HOST_HmServer:+$host Opt HMClient_upd $HOST_HmServer} 
  done

  return 0
}


#
#  usage: gen_deleteq <site> either Target or Source
#
gen_deleteq()
{
  if [ "$1" == "Target" ]; then
    hostlist=$(hostsin $HOST_EhServer_Target)
  elif [ "$1" == "Source" ]; then
    hostlist=$(hostsin $HOST_EhServer_Source)
  else
    msgprint $WARN "Unknown site $1.  Should either be Target or Source."
    msgprint2 378 $WARN $1
    return 1
  fi

  write_cmd text "# Delete $1 data queues"
  write_cmd dash

  for host in $hostlist; do
    q_dir=$(find_queues_dir $host)
    if [ "$q_dir" == "" ]; then
      msgprint $WARN "Failed to find Queues directory for $host."
      msgprint2 173 $WARN $host
      return 1
    fi

    write_cmd text "$host Opt rmqueue $q_dir" 
  done

  return 0
}


#
# start job on source Database server 
#   'Move events to History Database' one more time, to move events
# 
# start job on source Database server 
#   'TBSM Log Shipping Transaction Log Backup' one more time, to generate logs
#
# start job on source History server 
#   'TBSM Log Shipping Transaction Log Backup' one more time, to generate logs
#-----------------------------------------------------------------------------------
#
# start job on target Database server 
#   'TBSM Log Shipping Transaction Log Copy Job' one more time, to copy logs
#   'TBSM Log Shipping Transaction Log Load Job' one more time, to load logs
#
# start job on target History server 
#   'TBSM Log Shipping Transaction Log Copy Job' one more time, to copy logs
#
# start job on target History server 
#   'TBSM Log Shipping Transaction Log Copy Job' one more time, to copy logs
#   'TBSM Log Shipping Transaction Log Load Job' one more time, to load logs
#
# The starting of 'TBSM Log Shipping Transaction Log Copy Job' also depends on the availability of the 
#   source servers
#
gen_db_complete()
{
  write_cmd text "# Finalize Log Shipping"
  write_cmd dash

  # starting jobs on source
  #-------------------------- 
  if [ "$HOST_HistServer_Source" != "" ]; then
    write_cmd text "$HOST_DatabaseServer_Source Opt startjob 0 \"Move events to History Database\"" 
    write_cmd text "$HOST_HistServer_Source Opt startjob $WAIT_JOBDONE \"TBSM Log Shipping Transaction Log Backup\"" 
  fi

  write_cmd text "$HOST_DatabaseServer_Source Opt startjob $WAIT_JOBDONE \"TBSM Log Shipping Transaction Log Backup\"" 

  # starting jobs on target 
  #-------------------------- 
  write_cmd text "$HOST_DatabaseServer_Target Opt startCopyJob $WAIT_JOBDONE $HOST_DatabaseServer_Source \"TBSM Log Shipping Copy Job\"" 
  write_cmd text "$HOST_DatabaseServer_Target Opt dumptable master sysprocesses"
  write_cmd text "$HOST_DatabaseServer_Target Opt startjob $WAIT_JOBDONE \"TBSM Log Shipping Load Job\"" 

  if [ "$HOST_HistServer_Target" != "" ]; then
    write_cmd text "$HOST_HistServer_Target Opt startCopyJob $WAIT_JOBDONE $HOST_HistServer_Source \"TBSM Log Shipping Copy Job\"" 
    write_cmd text "$HOST_HistServer_Target Opt dumptable master sysprocesses"
    write_cmd text "$HOST_HistServer_Target Opt startjob $WAIT_JOBDONE \"TBSM Log Shipping Load Job\"" 
  fi
}


#
# enable db by running 'EXEC asisp_recover_databases'
#
gen_enable_db()
{
  # enable DB on target
  #--------------------
  write_cmd text "# Enable Target databases"
  write_cmd dash
  write_cmd text "$HOST_DatabaseServer_Target Req dbcmd msdb \"EXEC asisp_recover_databases\"" 

  if [ "$HOST_HistServer_Target" != "" ]; then
    write_cmd text "$HOST_HistServer_Target Req dbcmd msdb \"EXEC asisp_recover_databases @histserver=1\"" 
  fi
}


#
# generate command to 
#    update Object..MenuItem table
#    Invoke asisp_rename_sqlserver
#
gen_rename_sqlserver()
{
  write_cmd text "# Rename SQL Server "
  write_cmd dash

  if [ "$HOST_WebServer_Target" != "" ]; then
    write_cmd text "$HOST_DatabaseServer_Target Req upd_menuitem $HOST_WebServer_Source"
    sql_cmd="EXEC asisp_rename_sqlserver @db_host=N'$HOST_DatabaseServer_Target', @websvr_host=N'$HOST_WebServer_Target'"
  else
    sql_cmd="EXEC asisp_rename_sqlserver @db_host=N'$HOST_DatabaseServer_Target'"
  fi

  write_cmd text "$HOST_DatabaseServer_Target Req dbcmd master \"$sql_cmd\""
}


#
#  Generate command to update TGM Task Server
#
#
gen_upd_tgmtaskserver()
{
  write_cmd text "# Reset TGMTaskServer name on target DatabaseServer"
  write_cmd dash

  tgm_count=$(echo $HOST_TGMTaskServer_Source | awk '{print NF}')
  idx=1
  while [ $idx -le $tgm_count ]; do
    source_tgm=$(echo $HOST_TGMTaskServer_Source | awk -v idx=$idx '{print $idx}')
    target_tgm=$(echo $HOST_TGMTaskServer_Target | awk -v idx=$idx '{print $idx}')

    sql_cmd="UPDATE OS_A set _TGMTaskServer=N'$target_tgm' 
      WHERE UPPER(_TGMTaskServer)=N'$source_tgm'"
    write_cmd text "$HOST_DatabaseServer_Target Req dbcmd Object \"$sql_cmd\""

    idx=`expr $idx + 1`
  done
}


#
# generate command to update MVS Sender Service Map
#
gen_upd_sendersvcmap()
{
  write_cmd text "# Update MVS Sender Service Map"
  write_cmd dash

  for host in $(hostsin $HOST_SsServer_Target)
  do
    write_cmd text "$host Opt updatesendersvcmap $HOST_DatabaseServer_Target "
  done
}


#
# update the job list from Object database
# history server is not supported
#
gen_enablejobs()
{
  write_cmd text "# Import SQL Jobs "
  write_cmd dash

  # retrieve the job description from Object database
  #------------------------------------------------------
  sql_cmd="EXEC Object..asisp_importjobpackage failover"
  write_cmd text "$HOST_DatabaseServer_Target Opt dbcmd msdb \"$sql_cmd\""
   
  # enable and disable proper jobs
  #------------------------------------------------------
  sql_cmd="UPDATE msdb..sysjobs SET enabled=i.enabled FROM Object..msdb_sysjobs as i JOIN msdb..sysjobs as o ON (o.name=i.name) WHERE i.enabled=1 and i.name NOT LIKE 'TBSM Log Shipping %'" 

  write_cmd text "$HOST_DatabaseServer_Target Opt dbcmd msdb \"$sql_cmd\""
}


gen_register_stc()
{
  if [ "$HOST_SnaServer_Target" == "" ]; then
    return 0
  fi

  write_cmd text "# Register mainframe upload enabled STC "
  write_cmd dash
  write_cmd text "$HOST_DatabaseServer_Target Opt registerstc"
}


#
# copy data queues from source to target
#
gen_copyq()
{
  write_cmd text "# Copy data queues from Source to Target"
  write_cmd dash

  for host in $(hostsin $HOST_EhServer_Target)
  do
    # $HOST_EhServer_Source might be expanded to multiple hosts
    #-----------------------------------------------------------
    write_cmd text "$host Opt copyq $HOST_EhServer_Source"
  done
}


# 
# need to create a scheduled job to syn this file 
# the agentlistener properties file is located in 
#   $TivoliManager/AgentListener/config/<tecserver>.properties
gen_copy_agentlistener_file()
{
  write_cmd text "# Copy AgentListener TEC properties file from Source to Target"
  write_cmd dash
  write_cmd text ${HOST_AgentListener_Source:+$HOST_AgentListener_Source Opt copytecproperties $HOST_AgentListener_Target}
}



#-------------------------------------------------------------------------


# check network connectivity
# disable jobs on target
# stop services on target
# delete data queues on target
#----------------------------------------------
preprocess_fo()
{
  write_cmd blank
  write_cmd text "# Preprocess Failover"
  write_cmd dash

  # check if we can connect to source, this is optional 
  #----------------------------------------------------
  for host in $(hostsin $All_Source_Hosts) 
  do
    write_cmd text "$host Opt checkconnectivity"
  done

  # check if we can connect to non-required target
  #-----------------------------------------------
  hosts="$HOST_AgentListener_Target
    $HOST_CommonListener_Target
    $HOST_EhServer_Target
    $HOST_HistServer_Target
    $HOST_HmServer_Target
    $HOST_HmClient_Target
    $HOST_SnaServer_Target
    $HOST_SsServer_Target
    $HOST_EIFSender_Target
    $HOST_TECListener_Target
    $HOST_TGMTaskServer_Target
    $HOST_UpLoadRuleServer_Target
    $HOST_WebServer_Target
    $HOST_XMLToolkit_Target
    "
  for host in $(hostsin $hosts)
  do
    write_cmd text "$host Opt checkconnectivity"
  done

  # make sure we can connect to required target
  #------------------------------------------------------
  for host in $(hostsin $HOST_DatabaseServer_Target $HOST_WASServer_Target)
  do
    write_cmd text "$host Req checkconnectivity"
  done

  # disable SQL Jobs on Target
  #-----------------------------------
  sql_cmd="UPDATE sysjobs SET enabled=0 WHERE enabled=1"
  write_cmd text "$HOST_DatabaseServer_Target Opt dbcmd msdb \"$sql_cmd\""
  write_cmd text ${HOST_HistServer_Target:+$HOST_HistServer_Target Opt dbcmd msdb \"$sql_cmd\"}

  # make sure we can rename the target SQL Server name
  # SELECT * FROM sysobjects WHERE type = 'P' AND name = 'asisp_rename_sqlserver'
  #------------------------------------------------------
  write_cmd text "$HOST_DatabaseServer_Target Req dbchkexist master P asisp_rename_sqlserver"

  # stop services on target
  #------------------------
  gen_stopsvcs Target

  # delete data queues on target
  #-----------------------------
  gen_deleteq Target
  
  return 0
}


# configure services to disabled on source
# disable jobs on source
# stop services on source
# copy all non-empty data queues from source to target 
# copy AgentListener properties file from source to target
#-------------------------------------------------------------
stop_source()
{
  gen_configsvcs Source disabled

  # disable SQL Jobs on Source 
  #-----------------------------------
  sql_cmd="UPDATE sysjobs SET enabled=0 WHERE enabled=1"
  write_cmd text "$HOST_DatabaseServer_Source Opt dbcmd msdb \"$sql_cmd\""
  write_cmd text ${HOST_HistServer_Source:+$HOST_HistServer_Source Opt dbcmd msdb \"$sql_cmd\"}

  for host in $(hostsin $HOST_DatabaseServer_Source $HOST_HistServer_Source)
  do
    write_cmd text "$host Opt svcstop SQLServerAgent" 
    write_cmd text "$host Opt svcstart SQLServerAgent" 
  done

  gen_stopsvcs Source

  if [ "$NoSync" != TRUE ]; then
    gen_copyq
  else
    write_cmd text "# NoSync is on, Copy queue is skipped."
    write_cmd dash
  fi

  gen_copy_agentlistener_file
}


# reestablish live and history server connection
#   execute log backup job one more time on source
#   execute log copy job one more time on target
#   execute log Load job one more time on target
# update Reporting Server in MenuItem table
# update TGMTaskServer in OS_A table
# update SsServer in MVSSenderSvcMap table 
# configure services to auto on target
# start server on target 
# enable jobs on target
# register STC with data source 
#---------------------------------------------------
start_target()
{
  if [ "$NoSync" != TRUE ]; then
    gen_db_complete
    gen_enable_db
    gen_rename_sqlserver
    gen_upd_tgmtaskserver
    gen_upd_sendersvcmap
    gen_enablejobs Target
  else
    gen_enable_db
    gen_rename_sqlserver

    write_cmd text "# NoSync is turned on, database sync steps are skipped."
    write_cmd dash
  fi

  gen_configsvcs Target demand

  gen_startsvcs Target

  gen_register_stc
}


main()
{
  # load config info into memory
  #-----------------------------
  load_configinfo
  if [ $? != 0 ]; then
    msgprint $ERROR "Failed to load configuration information. Cannot continue!"
    msgprint2 182 $ERROR 
    exit 1
  fi

  msgprint $INFO "Successfully loaded Failover configuration!"
  msgprint2 347 $INFO 

  set_source_target $REVERSE

  # verify failover direction
  #-----------------------------
  if [ "$ForceDirection" == TRUE ]; then
    msgprint $INFO "Failover force direction mode is specified.  Direction is not checked!"
    msgprint2 209 $INFO 
  else

    is_readonly_db $HOST_DatabaseServer_Target
    dbstate=$?

    if [ "$dbstate" == 2 ]; then
      msgprint $ERROR "Unknown state of Target Database Server database. Cannot continue!"
      msgprint2 379 $ERROR 
      exit 1
    fi

    if [ "$dbstate" == 1 ]; then
      msgprint $ERROR "Incorrect failover direction specified. Cannot continue!"
      msgprint2 231 $ERROR 
      exit 1
    fi

    msgprint $INFO "Failover direction check returns OK!"
    msgprint2 208 $INFO 
  fi

  if [ "$NoSync" == TRUE ]; then
    msgprint $INFO "NoSync flag is turned on.  Database will not be synchronized!"
    msgprint2 266 $INFO 
  fi

  write_cmd begin Failover

  # step 1 - preprocessing
  #---------------------------
  preprocess_fo

  # step 2 - stop source
  #---------------------------
  stop_source

  # step 3 - start target  
  #---------------------------
  start_target

  write_cmd end Failover

  msgprint $INFO "Successfully generated command file in $FO_CMD_FILE"
  msgprint2 346 $INFO "$FO_CMD_FILE"

  # if Launch flag is set, execute the command file immediately
  #-------------------------------------------------------------
  if [ "$Launch" == TRUE ]; then

    msgprint $INFO "Launch flag is set, start executing command file $FO_CMD_FILE"
    msgprint2 247 $INFO "$FO_CMD_FILE"

    if [ "$Trace_flag" == TRUE ]; then
      fo_cmd_execute -T -f "$FO_CMD_FILE"
    else
      fo_cmd_execute -f "$FO_CMD_FILE"
    fi

    if [ $? != 0 ]; then
      return 1
    fi
  fi

  return 0
}

#----------------------------------------------------------------------------------------------
main
exit 0
