-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_moveevent.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.7
-- Date Modified: 07/04/09 14:12:11
--
-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It copies the event TABLE FROM the live server to the history server using BCP.
-- It also maintains a control value used by asisp_deleteoldevent FOR purging old event rows.

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id('asisp_moveevent') AND sysstat & 0xf = 4)
  DROP PROCEDURE asisp_moveevent
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_moveevent.sql, tbsm_base, tbsm_base31e
-- SID: 1.5.4.7
-- Date Modified: 07/04/09 14:12:11
-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It copies the event TABLE FROM the live server to the history server using BCP.
-- It also maintains a control value used by asisp_deleteoldevent FOR purging old event rows.

CREATE PROCEDURE asisp_moveevent
  @targetserv sysname,
  @bcpdir NVARCHAR(255),
  @targetdb sysname='History',
  @targettable sysname='eventhist',
  @frompassword NVARCHAR(30)='',
  @topassword   NVARCHAR(30)='',
  @fromsauser   NVARCHAR(30)='sa',
  @tosauser     NVARCHAR(30)='sa'
AS

  DECLARE @rtn INT, @server sysname, @cmd NVARCHAR(255), @targetservlc sysname
  DECLARE @tstamp BINARY(8)
  DECLARE @rc INT
  DECLARE @now NVARCHAR(40) --dcr117

  SET NOCOUNT ON

  -- make sure the SystemConfiguration flag indicates that the traditional reporting
  -- system is being used - this enables asisp_deleteoldevent to clean up based on
  -- the data received by the reporting system
  IF NOT EXISTS (SELECT 1 FROM SystemConfiguration
                   WHERE groupname = 'ReportingComponent'
                   AND   propertyname = 'TraditionalTBSM'
                   AND   propertyvalue = '1')
    EXEC _SetSystemConfiguration @groupname = 'ReportingComponent', 
         @propertyname = 'TraditionalTBSM', 
         @propertyvalue = '1', 
         @propertydatatype = 'TINYINT'

  SELECT @server=LOWER(@@servername)
  SELECT @targetservlc = LOWER(@targetserv)
  IF @frompassword=''
    SELECT @frompassword='sa_' + @server
  IF @topassword=''
    SELECT @topassword='sa_' + @targetservlc
  IF @fromsauser=''
    SELECT @fromsauser='sa'
  IF @tosauser=''
    SELECT @tosauser='sa'

-----
-- Phases I: Copy event TABLE to eventbcp TABLE
--Clear eventbcp
  TRUNCATE TABLE eventbcp

--Load all events INTO eventbcp exit IF none to process
  SELECT @tstamp=lastbcp_tstamp FROM event_lastbcp
  INSERT INTO eventbcp SELECT agent_cid,
                              agent_id,
                              agent_ctime,
                              agent_seqno,
                              obj_cid,
                              obj_id,
                              cid,
                              id,
                              name,
                              ctime,
                              detail,
                              alertstate,
                              priority,
                              state,
                              tstamp, 
                              '' --dcr117, a placeholder for extended long attribute
                              FROM event WITH (TABLOCKX) WHERE tstamp>@tstamp
  SELECT @rc = @@ROWCOUNT
  IF @rc=0
    RETURN 0

-- DCR117 
  -- update extended attribute of the eventbcp table 
  -- retrieve id, seq, and extended attribute and place the result into another table
  DECLARE @attrArray TABLE (row INT IDENTITY(1,1), id INT, seq INT, Attr NVARCHAR(3600))
  INSERT @attrArray
  SELECT eb.id, seq, value
  FROM ExtendedEventTecLongAttr e, ExtendedEvent_A a, eventbcp eb
  WHERE _ExtEvSlot00 = event_handle AND _ExtEvSlot01 = e.server_handle 
      AND _ExtEvSlot02 = e.date_reception AND _ExtEvSlot04 = e.event_class 
      AND e.keyword = 'IHSTTTEC_L'
      AND eb.id = a.id AND eb.cid = a.eventcid
  ORDER BY eb.id, seq

  SELECT @rc = @@ROWCOUNT
  IF @rc = 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_moveevent: No extended attribute found, no need to update extended_attr for eventbcp table', 0, 1, @now) WITH NOWAIT
  END
  ELSE
  BEGIN
    -- insert attributes into extended attribute column of eventbcp
    DECLARE @id INT
    DECLARE @attrPtr BINARY(16)
    DECLARE @attrVal NVARCHAR(3600) -- data type is the same as value column of ExtendedEventTecLongAttr table
    DECLARE @count INT
    SET @count = 1

    -- set all null values of extended_attr to '' before updating to avoid error later
    UPDATE eventbcp SET extended_attr = ''
    WHERE extended_attr IS NULL

    WHILE (@count <= @rc)
    BEGIN
      SELECT @attrPtr = TEXTPTR(eb.extended_attr), @attrVal = Attr FROM @attrArray a, eventbcp eb
      WHERE a.id = eb.id AND row = @count 
      UPDATETEXT eventbcp.extended_attr @attrPtr NULL 0 @attrVal
      SET @count = @count + 1
    END

  END -- end ELSE
--end dcr117

-----
-- Phase II: bcp the eventbcp TABLE out
  SELECT @cmd='bcp Object..eventbcp_V out "' + @bcpdir + '\temp_event.txt" -S' + @server + ' -U' + @fromsauser + ' -P' + @frompassword + ' -c'
  EXEC @rtn=master..xp_cmdshell @cmd, no_output
  IF @rtn<>0
  BEGIN
    RAISERROR ('Unable to bcp eventbcp table out.', 16, 1)
    RETURN(-3000)
  END

-----
-- Phase III: bcp the eventbcp TABLE INTO history
  SELECT @cmd='bcp ' + @targetdb + '..' + @targettable + ' IN "' + @bcpdir + '\temp_event.txt" -S' + @targetservlc + ' -U' + @tosauser + ' -P' + @topassword + ' -c -b 1000 -m 10000 -e "' + @bcpdir + '\temp_event.err"'
  EXEC @rtn=master..xp_cmdshell @cmd, no_output
  IF @rtn<>0
  BEGIN
    RAISERROR ('Unable to bcp into eventhist table in server=%s, database=%s', 16, 1, @targetservlc, @targetdb)
    RETURN(-3000)
  END

-----
-- Phase IV: UPDATE last BCP timestamp
  UPDATE event_lastbcp SET lastbcp_tstamp=(SELECT max(tstamp) FROM eventbcp)

  RETURN(0)
GO
