-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- Copyright IBM Corp. 2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/cl_registration.sql, tbsm.sql, tbsm_base31e
-- SID: 1.4.1.6
-- Date Modified: 07/04/20 16:02:19
--
-- Module Description
--   Defines the CL_Registration table

USE Object
GO

---------------------------------------------------------------------------------------------------
-- CL_Registration

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = N'CL_Registration' AND type = 'U')
BEGIN
  PRINT 'Creating table CL_Registration'
  CREATE TABLE CL_Registration (
	  registrationId 	int 			IDENTITY (1, 1) NOT NULL ,
	  instrId 		nvarchar (255) 		NOT NULL ,
	  instrAddress 		nvarchar (255) 		NOT NULL ,
	  sid 			nvarchar (255) 		NOT NULL
  )

  ALTER TABLE CL_Registration WITH NOCHECK ADD
	  CONSTRAINT DF_CL_Registration_sid DEFAULT (0) FOR sid,
	  CONSTRAINT PK_CL_Registration PRIMARY KEY  NONCLUSTERED
	  (
		registrationId
	  )  
END
GO

--
-- Add the internal TBSM registration to the registration table
--
IF NOT EXISTS (SELECT * FROM CL_Registration WHERE instrId='TBSM_LOCAL') BEGIN
  DECLARE @instrAddress NVARCHAR(510)
  DECLARE @sid          NVARCHAR(510)
  SET @instrAddress=@@SERVERNAME + '.BASETEST.QM+BASETEST.Q+9898'
  SET @sid='TBSM_LOCAL_INSTRUMENTATION ' + @@SERVERNAME + '-0000000000'
  INSERT INTO CL_Registration (instrId, instrAddress, sid) VALUES('TBSM_LOCAL_INSTRUMENTATION',@instrAddress,@sid)
END
GO

--
-- Add the instrName column
--
IF NOT EXISTS (SELECT * FROM syscolumns WHERE OBJECT_NAME(id) = 'CL_Registration' AND name = 'instrName')
BEGIN
  PRINT 'Adding CL_Registraion.instrName' 
  ALTER TABLE dbo.CL_Registration ADD instrName nvarchar(255) NULL
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tIU_Reg' AND type = 'TR')
  DROP TRIGGER tIU_Reg
GO
  
CREATE TRIGGER tIU_Reg ON CL_Registration AFTER INSERT,UPDATE
AS
  DECLARE @now NVARCHAR(40), @rc INT, @verbose TINYINT
  SET @verbose = 0

  IF @verbose >= 1
  BEGIN
    SELECT @rc = COUNT(*) FROM CL_Registration where instrName is NULL
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tIU_Reg: Starting. #rows=%d', 0, 1, @now, @rc) WITH NOWAIT
  END

  update CL_Registration set instrName = instrId + '@127.0.0.1' where instrName is null

  IF @verbose >= 1
  BEGIN
    SELECT @rc = COUNT(*) FROM CL_Registration where instrName is NULL
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tIU_Reg: Ending. #rows=%d', 0, 1, @now, @rc) WITH NOWAIT
  END

update CL_Registration set instrName = instrId + '@127.0.0.1' where instrName is null

